<?php
/**
 * Plugin Helpers FUnction
 *
 * @package WishGPT
 */

/**
 * Load Plugin Template File.
 *
 * @param string $template_file Template file path.
 * @param array  $variables  Variables.
 * @param bool   $include    Set false to return the content as string.
 *
 * @return string
 */
function wishgpt_template( string $template_file, array $variables = array(), bool $include = true ) : string {

	$template_path    = WISHGPT_PLUGIN_PATH . 'templates/' . $template_file . '.php';
	$template_content = '';

	if ( file_exists( $template_path ) ) {

		if ( ! $include ) {
			ob_start();
		}

		extract( $variables ); // phpcs:ignore

		include $template_path;

		if ( ! $include ) {
			$template_content = ob_get_clean();
		}
	}

	return $template_content;
}

/**
 * Get the Plugin Settings Option.
 *
 * @param string $option_key    Option Key.
 * @param mixed  $default_value Default Value.
 *
 * @return mixed
 */
function wishgpt_option( string $option_key, $default_value = '' ) {
	return get_option( 'wishgpt_' . $option_key, $default_value );
}

/**
 * Update the plugin settings option.
 *
 * @param string $option_key Option key.
 * @param mixed  $value  Value to update.
 *
 * @return bool
 */
function wishgpt_update_option( string $option_key, $value ) : bool {
	return update_option( 'wishgpt_' . $option_key, $value );
}

/**
 * Return available ChatGPT Models
 *
 * @return array
 */
function wishgpt_available_models() : array {
	$models = get_transient( 'wishgpt_models' );

	if ( empty( $models ) || ! is_array( $models ) ) {

		$models = array(
			wishgpt_defualt_model(),
		);

		// Set the API endpoint URL.
		$url = 'https://api.openai.com/v1/models';

		// Set the API key.
		$api_key = wishgpt_option( 'chatgpt_api_key' );

		if ( ! empty( $api_key ) ) {
			$response = wp_remote_get(
				$url,
				array(
					'headers' => array(
						'Authorization' => 'Bearer ' . $api_key,
					),
				)
			);

			// Check for errors.
			if ( is_wp_error( $response ) || empty( $response ) ) {
				return $models;
			}

			// Decode the JSON response.
			$response_data = json_decode( wp_remote_retrieve_body( $response ), true );

			if ( ! isset( $response_data['error'] ) && isset( $response_data['data'] ) ) {
				// Extract the model names from the response data.
				$models = array_filter( array_unique( array_column( $response_data['data'] ?? array(), 'id' ) ) );

				if ( ! empty( $models ) ) {
					set_transient( 'wishgpt_models', $models, 12 * HOUR_IN_SECONDS );
				}
			}
		}
	}

	return is_array( apply_filters( 'wishgpt_chatgpt_models_list', $models ) ) ? $models : array();
}

/**
 * Return Default ChatGPT Model
 *
 * @return string
 */
function wishgpt_defualt_model() : string {
	$default_model = 'text-davinci-002';
	$model         = apply_filters( 'wishgpt_chat_default_model', $default_model );
	return is_string( $model ) ? $model : $default_model;
}

/**
 * Generate Chat GPT Response bsaed on our provided Response.
 *
 * @param string $prompt    Prompt Text passed to Chat GPT.
 * @param string $model   Chat GPT Model.
 * @param int    $max_tokens   Maximum Tokens.
 *
 * @return string
 */
function wishgpt_chatgpt_api( string $prompt, string $model = 'OPTION', int $max_tokens = -1 ) : string {

	$output = '';

	if ( -1 === $max_tokens ) {
		$max_tokens = max( absint( wishgpt_option( 'chatgpt_max_tokens', 256 ) ), 100 );
	}
	if ( 'OPTION' === $model ) {
		$model = wishgpt_option( 'chatgpt_model', wishgpt_defualt_model() );
	}

	$api_key = wishgpt_option( 'chatgpt_api_key' );

	if ( empty( $api_key ) ) {
		return $output;
	}
	if ( empty( $model ) ) {
		$model = wishgpt_defualt_model();
	}
	$url  = 'https://api.openai.com/v1/completions';
	$data = array(
		'body'      => wp_json_encode(
			array(
				'prompt'     => $prompt,
				'max_tokens' => $max_tokens,
				'model'      => $model,
			)
		),
		'sslverify' => false,
		'timeout'   => 120,
		'headers'   => array(
			'Content-Type'  => 'application/json',
			'Authorization' => 'Bearer ' . $api_key,
		),
	);

	$response = wp_remote_post( $url, $data );

	if ( ! empty( $response ) && ! is_wp_error( $response ) && isset( $response['body'] ) ) {
		$response_body = json_decode( $response['body'], true );
		if ( ! empty( $response_body ) && isset( $response_body['choices'] ) && isset( $response_body['choices'][0] ) ) {
			$output = $response_body['choices'][0]['text'] ?? '';
		}
	}

	return ! is_string( $output ) ? '' : $output;

}

/**
 * Get client IP Address.
 *
 * @return string
 */
function wishgpt_get_client_ip() : string {
	if ( isset( $_SERVER['HTTP_X_REAL_IP'] ) ) {
		return sanitize_text_field( wp_unslash( $_SERVER['HTTP_X_REAL_IP'] ) );
	} elseif ( isset( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) {
		// Proxy servers can send through this header like this: X-Forwarded-For: client1, proxy1, proxy2
		// Make sure we always only send through the first IP in the list which should always be the client IP.
		return (string) rest_is_ip_address( trim( current( explode( '/,/', sanitize_text_field( wp_unslash( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) ) ) ) );
	} elseif ( isset( $_SERVER['REMOTE_ADDR'] ) ) {
		return sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) );
	}
	return '';
}

/**
 * Check if visitor can generate more wishes.
 *
 * @return bool|string
 */
function wishgpt_check_client_api_usage() {
	$max_generate = absint( wishgpt_option( 'max_generate', 0 ) );
	if ( $max_generate <= 0 ) {
		return true;
	}

	$client_ip = wishgpt_get_client_ip();
	$client_ip = str_replace( '.', '_', $client_ip );

	$usage = absint( get_transient( 'wishgpt_usage_' . $client_ip ) );
	if ( empty( $usage ) ) {
		$usage = absint( $_COOKIE['wishgpt_usage'] ?? 0 );
		if ( empty( $usage ) ) {
			$usage = $_SESSION['wishgpt_usage'] ?? 0;
		}
	}

	if ( $usage <= $max_generate ) {
		$update_usage              = $usage + 1;
		$_SESSION['wishgpt_usage'] = $update_usage;
		setcookie( 'wishgpt_usage', $update_usage, time() + 365 * 60 * 60 );
		set_transient( 'wishgpt_usage_' . $client_ip, $update_usage, 7 * DAY_IN_SECONDS );
		return true;
	}

	/* translators: %d Maxinum number of wishes a visitor can generate */
	return sprintf( esc_html__( 'You can generate maximum %d wishes.', 'wishgpt' ), $max_generate );
}
